@@ -3,7 +3,7 @@ use strict;
 use warnings;
 no warnings 'once';
 
-our $VERSION = 0.06;
+our $VERSION = 0.10;
 
 use Data::ParseBinary::Core;
 use Data::ParseBinary::Adapters;
@@ -245,29 +245,6 @@ sub Bitwise {
     return Restream($subcon, "Bit", "Bit");
 }
 
-my %library_types = (
-    'Graphics-BMP' => "Data::ParseBinary::lib::GraphicsBMP",
-    'Graphics-EMF' => "Data::ParseBinary::lib::GraphicsEMF",
-    'Graphics-PNG' => "Data::ParseBinary::lib::GraphicsPNG",
-    'Graphics-WMF' => "Data::ParseBinary::lib::GraphicsWMF",
-    'Executable-PE32' => "Data::ParseBinary::lib::ExecPE32",
-    'Executable-ELF32' => "Data::ParseBinary::lib::ExecELF32",
-    'Data-TermCapture' => "Data::ParseBinary::lib::DataCap",
-);
-
-sub Library {
-    my $type = pop @_;
-    die "Parse Library: Type not recognized" unless exists $library_types{$type};
-    my $type_name = $library_types{$type};
-    return $type_name if ref $type_name; # already loaded
-    eval qq{ require $type_name; };
-    die $@ if $@;
-    no strict 'refs';
-    my $type_ref = ${$type_name . '::Parser'};
-    $library_types{$type} = $type_ref;
-    return $type_ref;
-}
-
 sub Magic {
     my ($data) = @_;
     return Const(Field(undef, length($data)), $data);
@@ -603,10 +580,11 @@ It is also possible to have a default:
     $s = Enum(Byte("protocol"),
         TCP => 6,
         UDP => 17,
-        _default_ => "blah",
+        _default_ => blah => 99,
     );
     $s->parse("\x12") # returns 'blah'
 
+Please note that the default tag must not be one of the supplied pairs.
 And finally:
 
     $s = Enum(Byte("protocol"),
@@ -618,6 +596,21 @@ And finally:
 
 $DefaultPass tells Enum that if it isn't familiar with the value, pass it alone.
 
+If the field represent a set of flags, then the library provide a construct just for that:
+
+    $s = FlagsEnum(ULInt16("characteristics"),
+        RELOCS_STRIPPED => 0x0001,
+        EXECUTABLE_IMAGE => 0x0002,
+        LINE_NUMS_STRIPPED => 0x0004,
+        REMOVABLE_RUN_FROM_SWAP => 0x0400,
+        BIG_ENDIAN_MACHINE => 0x8000,
+    );
+    $data = $s->parse("\2\4");
+    # $data is { EXECUTABLE_IMAGE => 1, REMOVABLE_RUN_FROM_SWAP => 1 };
+
+Of course, this is equvalent to creating a BitStruct, and specifing Flag-s in the
+correct positions, and so on. but this is an easier way.
+
 We also have Validators. A Validator is an Adapter that instead of transforming data,
 validate it. Examples:
 
@@ -747,6 +740,21 @@ Anchor can help a Pointer to find it's target:
 Anchor saves the current location in the stream, enable the Pointer to jump to location
 relative to it.
 
+Optional construct may or may not be in the stream. Of course, it need a seekable stream.
+The optional section usually have a Const in them, that indicates is this section
+exists. 
+
+    my $wmf_file = Struct("wmf_file",
+        Optional(
+            Struct("placeable_header",
+                Const(ULInt32("key"), 0x9AC6CDD7),
+                ULInt16("handle"),
+            ),
+        ),
+        ULInt16("version"),
+        ULInt32("size"), # file size is in words
+    );
+
 =head2 Strings
 
 A string with constant length:
@@ -833,7 +841,55 @@ Copies "a" to "b".
     $data = $s->parse("\xaa\xbb\xcc\xdd");
     # $data is { a => 2864434397, b => 43707 }
 
-A Union. currently work only with primitives, and not on bit-stream.
+A Union. currently work only with constant-size constructs, (like primitives, Struct and such)
+but not on bit-stream.
+
+    $s = Struct("records",
+        ULInt32("record_size"),
+        RoughUnion("params",
+            Field("raw", sub { $_->ctx(1)->{record_size} - 8 }),
+            Array(sub { int(($_->ctx(1)->{record_size} - 8) / 4) }, ULInt32("params")),
+        ),
+    );
+
+RoughUnion is a type of Union, that doesn't check the size of it's sub-constructs.
+it is used when we don't know before-hand the size of the sub-constructs, and the size
+of the union as a whole. In the above example, we assume that if the union target is
+the array of integers, then it probably record_size % 4 = 0.
+
+If it's not, and we build this construct from the array, then we will be a few bytes
+short. 
+
+    $s = Struct("bmp",
+        ULInt32("width"),
+        ULInt32("height"),
+        Array(
+            sub { $_->ctx->{height} },
+            Aligned(
+                Array(
+                    sub { $_->ctx(2)->{width} },
+                    Byte("index")
+                ),
+            4),
+        ),
+    );
+
+Aligned make sure that the contained construct's size if dividable by $modulo. the
+syntex is:
+
+    Aligned($subcon, $modulo);
+
+In the above example, we have an excert from the BMP parser. each pixel is a byte.
+There is an array of lines (height) that each line is an array of pixels. each line
+is aligned to a four bytes boundary. 
+
+The modulo can be any number. 2, 4, 8, 7, 23. 
+
+    Magic("\x89PNG\r\n\x1a\n")
+
+A constant string that is written / read and verified to / from the stream.
+For example, every PNG file starts with eight pre-defined bytes. this construct
+handle them, transparant to the calling program.
 
 =head2 LasyBound
 
@@ -1060,67 +1116,19 @@ Reads from / Writes to a file. it is your responsebility to open the file and bi
 
 =head1 Format Library
 
-The Data::ParseBinary arrive with ever-expanding set of pre-defined parser for popular formats.
+The Data::ParseBinary arrive with ever-expanding set of pre-defined parsers for popular formats.
+Each of these parsers is in it's own module.
 And if you have a file-format, then this is how it's done:
 
-    my $bmp_parser = Data::ParseBinary->Library('Graphics-BMP');
+    use Data::ParseBinary::Graphics::BMP qw{$bmp_parser};
     open my $fh2, "<", $filename or die "can not open $filename";
     binmode $fh2;
     $data = $bmp_parser->parse(CreateStreamReader(File => $fh2));
 
 And $data will contain the parsed file. In the same way, it is possible to build a BMP file.
 
-The following explanations just highlight various issues with the various libraries.
-
-=head2 Graphics: BMP
-
-    my $bmp_parser = Data::ParseBinary->Library('Graphics-BMP');
-
-Can parse / build any BMP file, (1, 4, 8 or 24 bit) as long as RLE is not used.
-
-=head2 Graphics: EMF
-
-    my $emf_parser = Data::ParseBinary->Library('Graphics-EMF');
-
-This parser just do not work on my example file. Have to take a look on it.
-
-=head2 Graphics: PNG
-
-    my $png_parser = Data::ParseBinary->Library('Graphics-PNG');
-
-Parses the binay PNG format, however it does not decompress the compressed data.
-Also, it does not compute / verify the CRC values. 
-these actions are left to other layer in the program.
-
-=head2 Graphics: WMF
-
-    my $wmf_parser = Data::ParseBinary->Library('Graphics-WMF');
-
-No issues known.
-
-=head2 Executable: PE32
-
-    my $exec_pe32 = Data::ParseBinary->Library('Executable-PE32');
-
-Can parse a Windows (and DOS?) EXE and DLL files. However, when building it back,
-there are some minor differences from the original file, and Windows declare that
-it's not a valid Win32 application.
-
-=head2 Executable: ELF32
-
-    my $exec_elf32 = Data::ParseBinary->Library('Executable-ELF32');
-
-Can parse and re-build UNIX "so" files. 
-
-=head2 Data: Term Capture
-
-    my $data_cap = Data::ParseBinary->Library('Data-TermCapture');
-
-Parsing "tcpdump capture file", whatever it is. Please note that this parser
-have a lot of white space. (paddings) So when I rebuild the file, the padded
-area is zeroed, and the re-created file does not match the original file.
-
-I don't know if the recreated file is valid. 
+Please look for the documentation inside each module,
+as it highlights various issues with the various libraries.
 
 =head1 Debugging
 
@@ -1150,7 +1158,7 @@ Convert the original unit tests to Perl (and make them pass...)
 
 Fix the Graphics-EMF library
 
-Add documentation to: ExtractingAdapter, RoughUnion, Aligned, Magic, Optional
+Add documentation to: ExtractingAdapter
 
 Move the insertion of the parsed value to the context from the Struct/Sequence constructs
 to each indevidual construct?
@@ -1172,8 +1180,6 @@ Find out if the EMF file should work or not. it fails on the statment:
 Const(ULInt32("signature"), 0x464D4520)
 And complain that it gets "0".
 
-FlagsEnum: unit-test and document
-
 =head1 Thread Safety
 
 This is a pure perl module. there should be not problems.
